function interpolateDotCurrentEfficiency(rail,buckorldo) {
    let isBuck=buckorldo=='BUCK';
    if ((isBuck && !(Array.isArray(templateObj.config.BIoutData[rail-1]) && Array.isArray(templateObj.config.BEffData[rail-1]))) ||
        (!isBuck && !(Array.isArray(templateObj.config.BoostIoutData[rail-1]) && Array.isArray(templateObj.config.BoostEffData[rail-1])))) {
        return false;
    } else {
        // Get the indices for two closest currents to the user selected current
        let iout=0;
        let ioutData=0;
        let effData=0;
        if (isBuck) {
            iout=templateObj.config.BIout[rail-1];
            ioutData=templateObj.config.BIoutData[rail-1];
            effData=templateObj.config.BEffData[rail-1];
        } else {
            iout=templateObj.config.BoostIout[rail-1];
            ioutData=templateObj.config.BoostIoutData[rail-1];
            effData=templateObj.config.BoostEffData[rail-1];
        }
        let closest=findTwoClosestOneVariable(ioutData,iout);
        if (closest.CLOSE_NEG_INDEX==closest.CLOSE_POS_INDEX) {
            if (isBuck) {
                templateObj.config.BEff[rail-1]=effData[closest.CLOSE_NEG_INDEX];
            } else {
                templateObj.config.BoostEff[rail-1]=effData[closest.CLOSE_NEG_INDEX];
            }
        } else if (closest.CLOSE_NEG_INDEX==-1) {
            if (isBuck) {
                templateObj.config.BEff[rail-1]=effData[closest.CLOSE_POS_INDEX];
            } else {
                templateObj.config.BoostEff[rail-1]=effData[closest.CLOSE_POS_INDEX];
            }
        } else if (closest.CLOSE_POS_INDEX==-1) {
            if (isBuck) {
                templateObj.config.BEff[rail-1]=effData[closest.CLOSE_NEG_INDEX];
            } else {
                templateObj.config.BoostEff[rail-1]=effData[closest.CLOSE_NEG_INDEX];
            }
        } else {
            // Calculate the difference between the two closest currents
            let diff=ioutData[closest.CLOSE_POS_INDEX]-ioutData[closest.CLOSE_NEG_INDEX];
            // Caclculate the difference between the LOWER current and the target current.
            let targetDiffFromNeg=iout-ioutData[closest.CLOSE_NEG_INDEX];
            // Calculate the position of the target current between the two data points. Basically 0.5 would mean that 
            // the target current is exactly between the two data points.
            let relativeDifference=targetDiffFromNeg/diff;
            // Calculate the difference in efficiencies between the two closest current data points.
            let targetEffDiffFromNeg=effData[closest.CLOSE_POS_INDEX]-effData[closest.CLOSE_NEG_INDEX];
            // Do linear interpolation for the efficiency based on user set output current
            if (isBuck) {
                templateObj.config.BEff[rail-1]=effData[closest.CLOSE_NEG_INDEX]+targetEffDiffFromNeg*relativeDifference;
            } else {
                templateObj.config.BoostEff[rail-1]=effData[closest.CLOSE_NEG_INDEX]+targetEffDiffFromNeg*relativeDifference;
            }
        }
        return true;
    }
}

// data1 and data2 MUST have same IOUT and EFF length.
function interpolateEfficiencyGraph(data1,data2,variable,target) {
    let diff=data1[variable]-data2[variable]; // this should always be positive
    let targetDiffFromData2=target-data2[variable];
    let relativeDifference=targetDiffFromData2/diff;
    let interpolatedData={'Iout':data2.Iout,'Eff':[]};
    
    for (let i=0; i<data2.Eff.length; i++) {
        let pointDifference=data1.Eff[i]-data2.Eff[i];
        interpolatedData.Eff.push(data2.Eff[i]+pointDifference*relativeDifference);
    }
    return interpolatedData;
}

function findTwoClosestOneVariable(data,targetvariable,attribute='') {
    let closest_neg_match_index=-1;
    let closest_neg_diff=0;
    let closest_pos_match_index=-1;
    let closest_pos_diff=0;
    let diff=0;
    for (let i=0; i<data.length; i++) {
        if (attribute !== '') {
            diff=data[i][attribute]-targetvariable;
        } else {
            diff=data[i]-targetvariable;
        }
        // Exact match
        if (diff===0) {
            closest_neg_match_index=i;
            closest_neg_diff=diff;
            closest_pos_match_index=i;
            closest_pos_diff=diff;
            break; // No need to continue if we have exact match
        } else {
            if (diff < 0) { // Closest but smaller than the target value
                if (diff > closest_neg_diff || closest_neg_match_index==-1) {
                    closest_neg_match_index=i;
                    closest_neg_diff=diff;
                }
            } else { // Closest but larger than the target value
                if (diff < closest_pos_diff || closest_pos_match_index==-1) {
                    closest_pos_match_index=i;
                    closest_pos_diff=diff;
                } 
            }
        }
    }
    return {'CLOSE_POS_INDEX':closest_pos_match_index,'CLOSE_NEG_INDEX':closest_neg_match_index};
}

function getGraph(rail,vin,type='buck') {
    let success=true;
    let isBuck=type=='buck';
    let resultData={'Eff':[],'Iout':[]};
    let data = templateObj.config.device[type+'Data'].filter(function (element) {
        if (isBuck) {
            return element.VIN==vin &&
            element.FSW==templateObj.config.FSW[rail-1] &&
            element.FPWM==templateObj.config.FPWM[rail-1] &&
            element.phases==templateObj.config.phases[rail-1];
        } else {
            return element.VIN==vin &&
            element.FSW==templateObj.config.BoostFSW[rail-1]
        }
    });
    if (data.length>0) { // Check if we found data. Length is zero if not.
        let approximation=0;
        if (isBuck) {
            approximation=findTwoClosestOneVariable(data,templateObj.config.BVout[rail-1],'Vout');
        } else {
            approximation=findTwoClosestOneVariable(data,templateObj.config.BoostVout[rail-1],'Vout');
        }
        if (approximation.CLOSE_POS_INDEX==-1 || approximation.CLOSE_NEG_INDEX==-1) { // Outside the available data
            success=false;
        } else {
            // Closest positive and negative index are the same meaning, the value is exactly matched.
            if (approximation.CLOSE_POS_INDEX == approximation.CLOSE_NEG_INDEX) {
                // Not a VOUT approximation
                resultData.Eff=data[approximation.CLOSE_POS_INDEX].Eff;
                resultData.Iout=data[approximation.CLOSE_POS_INDEX].Iout;
            } else {
                if (isBuck) {
                    resultData=interpolateEfficiencyGraph(data[approximation.CLOSE_POS_INDEX],
                                    data[approximation.CLOSE_NEG_INDEX],
                                    'Vout',
                                    templateObj.config.BVout[rail-1]);
                } else {
                    resultData=interpolateEfficiencyGraph(data[approximation.CLOSE_POS_INDEX],
                                    data[approximation.CLOSE_NEG_INDEX],
                                    'Vout',
                                    templateObj.config.BoostVout[rail-1]);
                }
            }
        }
    } else {
        success=false;
    }
    if (success) {
        return {'Eff':resultData.Eff,'Iout':resultData.Iout,'VIN':vin};
    } else {
        // Set graphs to zero if no data available.
        return {'Eff':0,'Iout':0,'VIN':vin};
    }
}

function updateGraphs() {
    let lineColors=["#000000","#ff0000","#aaaaaa","#295c75","#286b49"];
    let chart=templateObj.efficiencyGraph._plotObj;
    
    let data=[];
    
    let labels=['','','','',''];
    let curveData=[[[0.0001,0.0001]],[[0.0001,0.0001]],[[0.0001,0.0001]],[[0.0001,0.0001]],[[0.0001,0.0001]]];
    let maxHorValue=0;
    
    for (let i=0; i<templateObj.config.device.buckCount+templateObj.config.device.boostCount; i++) {
        let current=0;
        let eff=0;
        let label='';
        let maxCurrent=0;
        // Branch for bucks
        if (i<templateObj.config.device.buckCount) {
            current=templateObj.config.BIoutData[i];
            eff=templateObj.config.BEffData[i];
            label='BUCK'+(i+1).toString()+' Efficiency';
            maxCurrent=templateObj.BIoutBX[i].maxValue;
        } else { // Boost
            current=templateObj.config.BoostIoutData[i-templateObj.config.device.buckCount];
            eff=templateObj.config.BoostEffData[i-templateObj.config.device.buckCount];
            label='BOOST'+(i-templateObj.config.device.buckCount+1).toString()+' Efficiency';
            maxCurrent=templateObj.BoostIoutBX[i-templateObj.config.device.buckCount].maxValue;
        }
        labels[i]=label;
        // Current and Eff vectors must be same length!
        for (let j=0; j<current.length; j++) {
            curveData[i].push([current[j],eff[j]]);
            if (current[j] > maxCurrent) {
                break;
            }
        }
        
        if (i<templateObj.config.device.buckCount) {
            if (templateObj.config.phases[i] != -1 && templateObj.BEnables[i].checked) {
                data.push({color:lineColors[i], lines: {show: true, fill: false}, data: curveData[i], label: labels[i]});
            }
        } else {
            if (templateObj.BoostEnables[i-templateObj.config.device.buckCount].checked) {
                data.push({color:lineColors[i], lines: {show: true, fill: false}, data: curveData[i], label: labels[i]});
            }
        }
    }
    chart.setData(data);
    chart.setupGrid(data);
    chart.draw();
}
